import { UUIHorizontalShakeKeyframes, UUIHorizontalShakeAnimationValue } from '@umbraco-ui/uui-base/lib/animations';
import { demandCustomElement } from '@umbraco-ui/uui-base/lib/utils';
import { LabelMixin } from '@umbraco-ui/uui-base/lib/mixins';
import { defineElement } from '@umbraco-ui/uui-base/lib/registration';
import { InterfaceLookDefaultValue } from '@umbraco-ui/uui-base/lib/types';
import { iconWrong, iconCheck } from '@umbraco-ui/uui-icon-registry-essential/lib/svgs';
import { LitElement, html, css } from 'lit';
import { property } from 'lit/decorators.js';

var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};
let UUIButtonElement = class extends LabelMixin("", LitElement) {
  constructor() {
    super();
    this.type = "button";
    this.disabled = false;
    this.look = InterfaceLookDefaultValue;
    this.compact = false;
    this.state = null;
    this._internals = this.attachInternals();
    this.addEventListener("click", this._onHostClick);
  }
  _onHostClick(e) {
    var _a;
    if (this.disabled) {
      e.preventDefault();
      e.stopImmediatePropagation();
      return;
    }
    if ((_a = this._internals) == null ? void 0 : _a.form) {
      switch (this.type) {
        case "reset":
          this._internals.form.reset();
          break;
        case "button":
          break;
        default:
          if (this._internals.form.requestSubmit) {
            this._internals.form.requestSubmit();
          } else {
            this._internals.form.dispatchEvent(new SubmitEvent("submit"));
          }
          break;
      }
    }
  }
  updated(changedProperties) {
    super.updated(changedProperties);
    if (changedProperties.has("state")) {
      clearTimeout(this._resetStateTimeout);
      if (this.state === "success" || this.state === "failed") {
        this._resetStateTimeout = setTimeout(() => this.state = null, 2e3);
      }
    }
  }
  renderState() {
    let element = html``;
    switch (this.state) {
      case "waiting":
        demandCustomElement(this, "uui-loader-circle");
        element = html`<uui-loader-circle id="loader"></uui-loader-circle>`;
        break;
      case "success":
        demandCustomElement(this, "uui-icon");
        element = html`<uui-icon
          name="check"
          .fallback=${iconCheck.strings[0]}></uui-icon>`;
        break;
      case "failed":
        demandCustomElement(this, "uui-icon");
        element = html`<uui-icon
          name="wrong"
          .fallback=${iconWrong.strings[0]}></uui-icon>`;
        break;
      default:
        return "";
    }
    return html`<div id="state">${element}</div>`;
  }
  render() {
    return html`
      <button ?disabled=${this.disabled} aria-label="${this.label}">
        ${this.renderState()} ${this.renderLabel()}
        <slot name="extra"></slot>
      </button>
    `;
  }
};
UUIButtonElement.styles = [
  UUIHorizontalShakeKeyframes,
  css`
      :host {
        position: relative;
        display: inline-block;
        margin-left: calc(var(--uui-button-merge-border-left, 0) * -1px);
        --uui-button-padding-left-factor: 3;
        --uui-button-padding-right-factor: 3;
        --uui-button-padding-top-factor: 1;
        --uui-button-padding-bottom-factor: 1;

        text-align: center;
        font-weight: var(
          --uui-button-font-weight,
          var(--uui-interface-font-weight,400)
        );
      }

      :host([compact]) {
        --uui-button-padding-left-factor: 1;
        --uui-button-padding-right-factor: 1;
      }

      .label {
        display: block;
        transition: opacity 120ms;
      }
      :host([state]:not([state=''])) .label {
        opacity: 0;
      }

      #state {
        position: absolute;
        transform: translate(-50%, -50%);
        top: 50%;
        left: 50%;
        opacity: 0;
        animation-name: fadeIn;
        animation-delay: 40ms;
        animation-duration: 360ms;
        animation-fill-mode: forwards;
      }

      button {
        height: 100%;
        min-height: var(--uui-button-height, auto);
        width: 100%;

        padding: calc(calc(8 / 15 * 1em) * var(--uui-button-padding-top-factor))
          calc(var(--uui-size-2,6px) * var(--uui-button-padding-right-factor))
          calc(calc(8 / 15 * 1em) * var(--uui-button-padding-bottom-factor))
          calc(var(--uui-size-2,6px) * var(--uui-button-padding-left-factor));
        text-align: inherit; /* Inherit from host so it can be overwritten from the outside */
        vertical-align: middle;
        box-shadow: none;
        border-width: var(--uui-button-border-width, 1px);
        border-style: solid;
        border-color: var(
          --uui-button-border-color,
          var(--uui-interface-surface,#fefefe)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-border-radius,3px)
        );
        cursor: pointer;

        font-size: var(--uui-button-font-size, inherit);
        font-family: inherit;
        font-weight: inherit; /* Inherit from host so it can be overwritten from the outside */

        background-color: var(
          --uui-button-background-color,
          var(--uui-interface-surface,#fefefe)
        );
        color: var(--uui-button-contrast, var(--uui-interface-contrast,#060606));

        transition: background-color 80ms, border-color 80ms, color 80ms;
      }
      button:hover {
        background-color: var(
          --uui-button-background-color-hover,
          var(--uui-interface-surface-hover,rgb(250, 250, 250))
        );
        border-color: var(
          --uui-button-border-color-hover,
          var(--uui-interface-surface-hover,rgb(250, 250, 250))
        );
        color: var(
          --uui-button-contrast-hover,
          var(--uui-interface-contrast-hover,#1b264f)
        );
      }

      button[disabled] {
        background-color: var(
          --uui-button-background-color-disabled,
          var(--uui-interface-surface-disabled,rgb(228, 228, 228))
        );
        border-color: var(
          --uui-button-border-color-disabled,
          var(--uui-interface-surface-disabled,rgb(228, 228, 228))
        );
        color: var(
          --uui-button-contrast-disabled,
          var(--uui-interface-contrast-disabled,rgb(115, 113, 110))
        );
        cursor: default;
      }
      button[disabled]:active {
        animation: ${UUIHorizontalShakeAnimationValue};
      }
      #icon-check,
      #icon-wrong {
        fill: currentColor;
        display: grid;
        place-items: center;
        width: 1.5em;
      }

      #loader {
        font-size: 1.5em;
      }
      :host([look]:not([look=''])) #loader {
        color: inherit;
      }

      /* ANIMATIONS */
      @keyframes fadeIn {
        0% {
          opacity: 0;
        }
        100% {
          opacity: 1;
        }
      }

      @keyframes fadeOut {
        0% {
          opacity: 1;
        }
        100% {
          opacity: 0;
        }
      }

      /* LOOKS */

      :host([look='primary']) {
        font-weight: var(--uui-look-primary-font-weight,700);
      }
      :host([look='primary']) button {
        background-color: var(
          --uui-button-background-color,
          var(--uui-look-primary-surface,#1b264f)
        );
        color: var(--uui-look-primary-contrast,#fefefe);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-primary-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-primary-border-radius, var(--uui-border-radius,3px))
        );
        border-color: var(
          --uui-button-border-color,
          var(--uui-look-primary-border,#1b264f)
        );
      }
      :host([look='primary']) button:hover {
        background-color: var(
          --uui-button-background-color-hover,
          var(--uui-look-primary-surface-hover,#3544b1)
        );
        color: var(--uui-look-primary-contrast-hover,#fefefe);
        border-color: var(
          --uui-button-border-color-hover,
          var(--uui-look-primary-border-hover,#3544b1)
        );
      }
      :host([look='primary']) button[disabled] {
        background-color: var(
          --uui-button-background-color-disabled,
          var(--uui-look-primary-surface-disabled,rgb(115, 120, 130))
        );
        color: var(--uui-look-primary-contrast-disabled,rgb(228, 228, 228));
        border-color: var(
          --uui-button-border-color-disabled,
          var(--uui-look-primary-border-disabled,rgb(115, 120, 130))
        );
      }

      :host([look='secondary']) {
        font-weight: var(--uui-look-secondary-font-weight,500);
      }
      :host([look='secondary']) button {
        background-color: var(
          --uui-button-background-color,
          var(--uui-look-secondary-surface,#e2dad4)
        );
        color: var(--uui-look-secondary-contrast,#1b264f);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-secondary-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-secondary-border-radius, var(--uui-border-radius,3px))
        );
        border-color: var(
          --uui-button-border-color,
          var(--uui-look-secondary-border,#e2dad4)
        );
      }
      :host([look='secondary']) button:hover {
        background-color: var(
          --uui-button-background-color-hover,
          var(--uui-look-secondary-surface-hover,rgb(243, 239, 237))
        );
        color: var(--uui-look-secondary-contrast-hover,#3544b1);
        border-color: var(
          --uui-button-border-color-hover,
          var(--uui-look-secondary-border-hover,rgb(243, 239, 237))
        );
      }
      :host([look='secondary']) button[disabled] {
        background-color: var(
          --uui-button-background-color-disabled,
          var(--uui-look-secondary-surface-disabled,rgb(207, 207, 207))
        );
        color: var(--uui-look-secondary-contrast-disabled,rgb(106, 117, 154));
        border-color: var(
          --uui-button-border-color-disabled,
          var(--uui-look-secondary-border-disabled,rgb(207, 207, 207))
        );
      }

      :host([look='outline']) button {
        font-weight: var(--uui-look-outline-font-weight,700);
      }
      :host([look='outline']) button {
        background-color: var(
          --uui-button-background-color,
          var(--uui-look-outline-surface,transparent)
        );
        color: var(--uui-look-outline-contrast,#1b264f);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-outline-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-outline-border-radius, var(--uui-border-radius,3px))
        );
        border-color: var(
          --uui-button-border-color,
          var(--uui-look-outline-border,#1b264f)
        );
      }
      :host([look='outline']) button:hover {
        background-color: var(
          --uui-button-background-color-hover,
          var(--uui-look-outline-surface-hover,transparent)
        );
        color: var(--uui-look-outline-contrast-hover,#3544b1);
        border-color: var(
          --uui-button-border-color-hover,
          var(--uui-look-outline-border-hover,#3544b1)
        );
      }
      :host([look='outline']) button[disabled] {
        background-color: var(
          --uui-button-background-color-disabled,
          var(--uui-look-outline-surface-disabled,transparent)
        );
        color: var(--uui-look-outline-contrast-disabled,rgb(106, 117, 154));
        border-color: var(
          --uui-button-border-color-disabled,
          var(--uui-look-outline-border-disabled,rgb(106, 117, 154))
        );
      }

      :host([look='placeholder']) {
        font-weight: var(--uui-look-placeholder-font-weight,400);
      }
      :host([look='placeholder']) button {
        background-color: var(
          --uui-button-background-color,
          var(--uui-look-placeholder-surface,transparent)
        );
        color: var(--uui-look-placeholder-contrast,#1b264f);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-placeholder-border-style, dashed)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-placeholder-border-radius, var(--uui-border-radius,3px))
        );
        border-color: var(
          --uui-button-border-color,
          var(--uui-look-placeholder-border,#1b264f)
        );
      }
      :host([look='placeholder']) button:hover {
        background-color: var(
          --uui-button-background-color-hover,
          var(--uui-look-placeholder-surface-hover,transparent)
        );
        color: var(--uui-look-placeholder-contrast-hover,#3544b1);
        border-color: var(
          --uui-button-border-color-hover,
          var(--uui-look-placeholder-border-hover,#3544b1)
        );
      }
      :host([look='placeholder']) button[disabled] {
        background-color: var(
          --uui-button-background-color-disabled,
          var(--uui-look-placeholder-surface-disabled,transparent)
        );
        color: var(--uui-look-placeholder-contrast-disabled,rgb(106, 117, 154));
        border-color: var(
          --uui-button-border-color-disabled,
          var(--uui-look-placeholder-border-disabled,rgb(106, 117, 154))
        );
      }

      :host([look='positive']) {
        font-weight: var(--uui-look-positive-font-weight,700);
      }
      :host([look='positive']) button {
        background-color: var(
          --uui-button-background-color,
          var(--uui-look-positive-surface,#25aa60)
        );
        color: var(--uui-look-positive-contrast,#fefefe);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-positive-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-positive-border-radius, var(--uui-border-radius,3px))
        );
        border-color: var(
          --uui-button-border-color,
          var(--uui-look-positive-border,#25aa60)
        );
      }
      :host([look='positive']) button:hover {
        background-color: var(
          --uui-button-background-color-hover,
          var(--uui-look-positive-surface-hover,rgb(43, 197, 112))
        );
        color: var(--uui-look-positive-contrast-hover,#fefefe);
        border-color: var(
          --uui-button-border-color-hover,
          var(--uui-look-positive-border-hover,rgb(43, 197, 112))
        );
      }
      :host([look='positive']) button[disabled] {
        background-color: var(
          --uui-button-background-color-disabled,
          var(--uui-look-positive-surface-disabled,rgb(89, 117, 102))
        );
        color: var(--uui-look-positive-contrast-disabled,#fefefe);
        border-color: var(
          --uui-button-border-color-disabled,
          var(--uui-look-positive-border-disabled,rgb(89, 117, 102))
        );
      }

      :host([look='warning']) {
        font-weight: var(--uui-look-warning-font-weight,700);
      }
      :host([look='warning']) button {
        background-color: var(
          --uui-button-background-color,
          var(--uui-look-warning-surface,#fad634)
        );
        color: var(--uui-look-warning-contrast,#060606);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-warning-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-warning-border-radius, var(--uui-border-radius,3px))
        );
        border-color: var(
          --uui-button-border-color,
          var(--uui-look-warning-border,#fad634)
        );
      }
      :host([look='warning']) button:hover {
        background-color: var(
          --uui-button-background-color-hover,
          var(--uui-look-warning-surface-hover,rgb(251, 224, 101))
        );
        color: var(--uui-look-warning-contrast-hover,#060606);
        border-color: var(
          --uui-button-border-color-hover,
          var(--uui-look-warning-border-hover,rgb(251, 224, 101))
        );
      }
      :host([look='warning']) button[disabled] {
        background-color: var(
          --uui-button-background-color-disabled,
          var(--uui-look-warning-surface-disabled,rgb(166, 160, 133))
        );
        color: var(--uui-look-warning-contrast-disabled,#060606);
        border-color: var(
          --uui-button-border-color-disabled,
          var(--uui-look-warning-border-disabled,rgb(166, 160, 133))
        );
      }

      :host([look='danger']) {
        font-weight: var(--uui-look-danger-font-weight,700);
      }
      :host([look='danger']) button {
        background-color: var(
          --uui-button-background-color,
          var(--uui-look-danger-surface,#d42054)
        );
        color: var(--uui-look-danger-contrast,#fefefe);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-danger-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-danger-border-radius, var(--uui-border-radius,3px))
        );
        border-color: var(
          --uui-button-border-color,
          var(--uui-look-danger-border,#d42054)
        );
      }
      :host([look='danger']) button:hover {
        background-color: var(
          --uui-button-background-color-hover,
          var(--uui-look-danger-surface-hover,rgb(226, 60, 107))
        );
        color: var(--uui-look-danger-contrast-hover,#fefefe);
        border-color: var(
          --uui-button-border-color-hover,
          var(--uui-look-danger-border-hover,rgb(226, 60, 107))
        );
      }
      :host([look='danger']) button[disabled] {
        background-color: var(
          --uui-button-background-color-disabled,
          var(--uui-look-danger-surface-disabled,rgb(133, 107, 114))
        );
        color: var(--uui-look-danger-contrast-disabled,#fefefe);
        border-color: var(
          --uui-button-border-color-disabled,
          var(--uui-look-danger-border-disabled,rgb(133, 107, 114))
        );
      }
    `
];
UUIButtonElement.formAssociated = true;
__decorateClass([
  property({ type: String, reflect: true })
], UUIButtonElement.prototype, "type", 2);
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIButtonElement.prototype, "disabled", 2);
__decorateClass([
  property({ reflect: true })
], UUIButtonElement.prototype, "look", 2);
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIButtonElement.prototype, "compact", 2);
__decorateClass([
  property({ type: String, reflect: true })
], UUIButtonElement.prototype, "state", 2);
UUIButtonElement = __decorateClass([
  defineElement("uui-button")
], UUIButtonElement);

export { UUIButtonElement };
